import torch
import torch.nn as nn
import torch.nn.functional as F
from densenet2 import densenet121,densenet169
from sklearn.metrics import roc_auc_score
class Cls_model(nn.Module):
    def __init__(self, n_channels=2, n_classes=1):
        super(Cls_model, self).__init__()
        self.backbone1 = densenet121(pretrained=False)  #1024
        self.backbone2 = densenet169(pretrained=False)  #1664

        self.pool = nn.AdaptiveAvgPool2d((1,1))
        self.fc = nn.Linear(1024*3,15)
        self.trans_layer1 = nn.Sequential(
            nn.ReLU(True),
            nn.Conv2d(1024,1024,1)
        )
        self.trans_layer2 = nn.Sequential(
            nn.ReLU(True),
            nn.Conv2d(1664, 1024, 1)
        )
    def forward(self, x):
        x1 = self.backbone1.get_feature(x)
        x2 = self.backbone2.get_feature(x)
        x1 = self.trans_layer1(x1)
        x2 = self.trans_layer2(x2)
        amap1=x1.sum(dim=1)
        amap2 = x2.sum(dim=1)
        x3 = x1 * x2
        x = torch.cat([x1,x3,x2],dim=1)
        x = self.pool(x)
        x = x.squeeze(-1).squeeze(-1)
        x = self.fc(x)
        return x,amap1,amap2
