import torch
import torch.nn as nn
import torch.nn.functional as F
import densenet4
from densenet4 import densenet121,densenet169
from sklearn.metrics import roc_auc_score

class ms_cam(nn.Module):
    def __init__(self,in_c,r,BatchNorm2d=nn.BatchNorm2d):
        super(ms_cam,self).__init__()
        self.branch1= nn.Sequential(nn.AdaptiveAvgPool2d((1,1)),
                                    nn.Conv2d(in_c,in_c//r,kernel_size=1,stride=1,bias=False),
                                    BatchNorm2d(in_c//r),
                                    nn.ReLU(),
                                    nn.Conv2d(in_c // r, in_c, kernel_size=1, stride=1, bias=False),
                                    BatchNorm2d(in_c),
                                    )

        self.branch2 = nn.Sequential(
            nn.Conv2d(in_c,in_c//r,kernel_size=1,stride=1,bias=False),
            BatchNorm2d(in_c // r),
            nn.ReLU(),
            nn.Conv2d(in_c//r, in_c, kernel_size=1, stride=1, bias=False),
            BatchNorm2d(in_c)
        )
    def forward(self,x):

        x1 = self.branch1(x)
        x2 = self.branch2(x)
        x3 = x1 + x2
        x3 = torch.sigmoid(x3)

        return x3

class iAFF(nn.Module):
    def __init__(self,in_c,r,BatchNorm2d=nn.BatchNorm2d):
        super(iAFF,self).__init__()
        self.ms_cam1 = ms_cam(in_c,r,BatchNorm2d=BatchNorm2d)
        self.ms_cam2 = ms_cam(in_c, r,BatchNorm2d=BatchNorm2d)

    def forward(self,x_1,x_2):
        x1_sum= x_1 + x_2
        amap1 = self.ms_cam1(x1_sum)

        x_1_a = amap1 * x_1
        x_2_a = (1 - amap1) * x_2

        x2_sum = x_1_a + x_2_a
        amap2 = self.ms_cam2(x2_sum)

        x_1 = amap2 * x_1
        x_2 = (1 - amap2) * x_2

        x = x_1 + x_2
        return x

class ms_fusion(nn.Module):
    def __init__(self,in_c,out_c,r,BatchNorm2d=nn.BatchNorm2d):
        super(ms_fusion,self).__init__()
        self.iaff = iAFF(out_c,r,BatchNorm2d=BatchNorm2d)
        self.down_sample = nn.Sequential(
            nn.Conv2d(in_c,out_c,kernel_size=3,padding=1,stride=2,bias=False),
            nn.BatchNorm2d(out_c)
        )

    def forward(self,x_1,x_2):
        x_1 = self.down_sample(x_1)
        x = self.iaff(x_1,x_2)
        return x

class Cls_model(nn.Module):
    def __init__(self, n_channels=2, n_classes=1,BatchNorm2d=nn.BatchNorm2d):
        super(Cls_model, self).__init__()
        densenet4.BatchNorm2d = BatchNorm2d
        self.backbone1 = densenet121(pretrained=False)  #1024

        # torch.Size([2, 256, 56, 56])
        # torch.Size([2, 512, 28, 28])
        # torch.Size([2, 1024, 14, 14])
        # torch.Size([2, 1024, 7, 7])
        self.ms_fusion1 = ms_fusion(256,512,8,BatchNorm2d=BatchNorm2d)
        self.ms_fusion2 = ms_fusion(512,1024,8,BatchNorm2d=BatchNorm2d)
        self.ms_fusion3 = ms_fusion(1024,1024,8,BatchNorm2d=BatchNorm2d)

        self.pool = nn.AdaptiveAvgPool2d((1,1))
        self.fc = nn.Linear(1024+1024+512+256,1024)
        self.relu = nn.ReLU()
        self.fc_end = nn.Linear(1024,14)

    def forward(self, x):
        x1,x2,x3,x4 = self.backbone1._forward(x)

        feat1 = self.pool(x1)
        feat1 = feat1.squeeze(-1).squeeze(-1)
        feat2 = self.pool(x2)
        feat2 = feat2.squeeze(-1).squeeze(-1)
        feat3 = self.pool(x3)
        feat3 = feat3.squeeze(-1).squeeze(-1)
        feat4 = self.pool(x4)
        feat4 = feat4.squeeze(-1).squeeze(-1)
        x = self.fc(torch.cat([feat1,feat2,feat3,feat4],dim=1))
        x = self.relu(x)
        x = self.fc_end(x)
        return x

if __name__ == '__main__':
    x = torch.rand([2,1,224,224])
    model = Cls_model()
    y = model(x)
    print(y.size())
