import torch
import torch.utils.data as data
import pandas as pd
import cv2
import numpy as np
import random
import os
root_img='/home/xujing/chestmcls/data/data_imgs/images'
path_list=r'D:\myfiles\code\others2\seg_cls\dataset\fake_label'
SIZE=256
CROP_SIZE=224

def load_img(path):
    img = cv2.imread(path,0)
    img = np.array(img, dtype="float") / 255.0
    return img

# def normalization(img,means=means,stds=stds):
#     for i in range(img.shape[-1]):
#         img[:,:,i] =  (img[:,:,i] - means[i])/stds[i]
#     return img
def load_labels(labels,num_classes=15):
    label = np.zeros([num_classes],np.uint8)
    nums = labels.strip().split(';')[:-1]
    for num in nums:
        label[int(num)] = 1
    return label[1:]

def default_loader(path):
    img = load_img(path)
    img = cv2.resize(img,(SIZE,SIZE))
    flip_p = random.random()
    rotate_p = random.random()
    if(rotate_p>0.5):
        img = np.rot90(img)
    img = np.expand_dims(img, 0)
    if (flip_p < 0.25):
        pass
    elif (flip_p < 0.5):
        for k in range(1):
            img[k, :, :] = np.fliplr(img[k, :, :])
    elif (flip_p < 0.75):
        for k in range(1):
            img[k, :, :] = np.flipud(img[k, :, :])
    elif (flip_p < 1.0):
        for k in range(1):
            img[k, :, :] = np.fliplr(np.flipud(img[k, :, :]))
    # mask=np.expand_dims(mask,axis=0)
    # img = np.expand_dims(img, axis=0)
    return  img

def default_loader_val(path):
    img = load_img(path)
    img = cv2.resize(img, (SIZE, SIZE))
    img = np.expand_dims(img, axis=0)
    return  img


class ImageFolder(data.Dataset):

    def __init__(self,path):
        self.ids =  pd.read_csv(path,header=None,sep=',').values
        self.loader = default_loader
        self.label_loader = load_labels

    def __getitem__(self, index):
        path,label = self.ids[index]
        label = self.label_loader(label)
        img = self.loader(os.path.join(root_img,path))
        img = np.ascontiguousarray(img, dtype=np.float32)
        img = torch.Tensor(img)
        label = torch.Tensor(label).float()
        return img,label

    def __len__(self):
        return len(self.ids)
        # return 640 * 2

class ImageFolder_val(data.Dataset):

    def __init__(self,path):
        self.ids = pd.read_csv(path,header=None,sep=',').values
        self.loader = default_loader_val
        self.label_loader = load_labels
    def __getitem__(self, index):
        path,label = self.ids[index]
        label = self.label_loader(label)
        img = self.loader(os.path.join(root_img, path))
        img = np.ascontiguousarray(img, dtype=np.float32)
        img = torch.Tensor(img)
        label = torch.Tensor(label).float()
        return img,label

    def __len__(self):
        return len(self.ids)
        # return 640*2

